mod conn;
pub mod entities;
mod migrator;

use migrator::Migrator;
use sea_orm::ColumnTrait;
use sea_orm::ConnectOptions;
use sea_orm::Database;
use sea_orm::DatabaseConnection;
use sea_orm::EntityTrait;
use sea_orm::PaginatorTrait;
use sea_orm::QueryFilter;
use sea_orm::QueryOrder;
use sea_orm_migration::MigratorTrait;

pub use entities::prelude::*;
pub use entities::*;

type Result<T> = std::result::Result<T, sea_orm::DbErr>;

#[derive(Clone, Debug)]
pub struct RieterDb {
    pub conn: DatabaseConnection,
}

impl RieterDb {
    pub async fn connect<C: Into<ConnectOptions>>(opt: C) -> Result<Self> {
        let db = Database::connect(opt).await?;

        Migrator::up(&db, None).await?;

        Ok(Self { conn: db })
    }

    pub async fn repos(&self, per_page: u64, page: u64) -> Result<(u64, Vec<repo::Model>)> {
        let paginator = Repo::find()
            .order_by_asc(repo::Column::Id)
            .paginate(&self.conn, per_page);
        let repos = paginator.fetch_page(page).await?;
        let total_pages = paginator.num_pages().await?;

        Ok((total_pages, repos))
    }

    pub async fn repo(&self, id: i32) -> Result<Option<repo::Model>> {
        repo::Entity::find_by_id(id).one(&self.conn).await
    }

    pub async fn repo_by_name(&self, name: &str) -> Result<Option<repo::Model>> {
        Repo::find()
            .filter(repo::Column::Name.eq(name))
            .one(&self.conn)
            .await
    }

    pub async fn packages(&self, per_page: u64, page: u64) -> Result<(u64, Vec<package::Model>)> {
        let paginator = Package::find()
            .order_by_asc(package::Column::Id)
            .paginate(&self.conn, per_page);
        let packages = paginator.fetch_page(page).await?;
        let total_pages = paginator.num_pages().await?;

        Ok((total_pages, packages))
    }

    pub async fn package(&self, id: i32) -> Result<Option<package::Model>> {
        package::Entity::find_by_id(id).one(&self.conn).await
    }
}
